#pragma once

#include "Block.h"
#include "BlockType.h"
#include "Item.h"
#include "TypeRegistry.h"

class BlockType;
class ItemType;
class BasicBlockType;

class BasicBlock : public Block
{
public:
    BasicBlock(int typeId, Framework::Vec3<int> pos, int dimensionId);
    BasicBlock(int typeId,
        Framework::Vec3<int> pos,
        int dimensionId,
        bool hasInventory);
    virtual bool onTick(
        TickQueue* zQueue, int numTicks, bool& blocked) override;
    virtual void onPostTick() override;

    friend BasicBlockType;
};

struct SpawnConfig
{
    int min;
    int max;
    double chance;
    Framework::Text itemTypeName;
    int typeId;
};

class BasicBlockType : public BlockType
{
private:
    Framework::Text itemTypeName;
    int itemTypeId;
    bool transparent;
    bool passable;
    float speedModifier;
    bool interactable;

protected:
    virtual void createSuperBlock(Block* zBlock, Item* zItem) const override;

public:
    BasicBlockType();
    virtual bool initialize(Game* zGame) override;
    virtual Block* createBlock(
        Framework::Vec3<int> position, int dimensionId) const override;
    virtual Item* createItem() const override;
    Framework::Text getItemTypeName() const;
    virtual ItemType* createItemType() const override;
    void setItemTypeName(Framework::Text itemTypeName);
    int getItemTypeId() const;
    void setTransparent(bool transparent);
    bool isTransparent() const;
    void setPassable(bool passable);
    bool isPassable() const;
    void setSpeedModifier(float speedModifier);
    float getSpeedModifier() const;
    void setInteractable(bool interactable);
    bool isInteractable() const;
};

class BasicBlockTypeFactory : public BlockTypeFactoryBase<BasicBlockType>
{
public:
    BasicBlockTypeFactory();
    virtual BasicBlockType* createValue(
        Framework::JSON::JSONObject* zJson) const override;
    virtual BasicBlockType* fromJson(
        Framework::JSON::JSONObject* zJson) const override;
    virtual Framework::JSON::JSONObject* toJsonObject(
        BasicBlockType* zObject) const override;
    virtual JSONObjectValidationBuilder* addToValidator(
        JSONObjectValidationBuilder* builder) const override;
    virtual const char* getTypeToken() const override;
};

class AdditionalItemSpawningBlock : public BasicBlock
{
private:
    Framework::Array<SpawnConfig> spawns;

public:
    AdditionalItemSpawningBlock(
        int typeId, Framework::Vec3<int> pos, int dimensionId);
    void addSpawn(SpawnConfig config);
    virtual void onDestroy() override;
};

class AdditionalItemSpawningBlockType : public BasicBlockType
{
private:
    Framework::Array<SpawnConfig> spawns;

public:
    AdditionalItemSpawningBlockType();

protected:
    virtual void createSuperBlock(Block* zBlock, Item* zItem) const override;

public:
    virtual bool initialize(Game* zGame) override;
    virtual Block* createBlock(
        Framework::Vec3<int> position, int dimensionId) const override;
    void addSpawn(SpawnConfig config);
    const Framework::Array<SpawnConfig>& getSpawns() const;
};

class AdditionalItemSpawningBlockTypeFactory : public BasicBlockTypeFactory
{
public:
    AdditionalItemSpawningBlockTypeFactory();
    BasicBlockType* createValue(
        Framework::JSON::JSONObject* zJson) const override;
    BasicBlockType* fromJson(Framework::JSON::JSONObject* zJson) const override;
    Framework::JSON::JSONObject* toJsonObject(
        BasicBlockType* zObject) const override;
    JSONObjectValidationBuilder* addToValidator(
        JSONObjectValidationBuilder* builder) const override;
    const char* getTypeToken() const override;
    const char* getTypeName() const override;
};